package net.sf.javaprinciples.data.transformer;

import java.util.Collection;

import net.sf.javaprinciples.core.UnexpectedException;
import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * The mapper will loop through each attribute and use the model name method to find the name of each attribute.
 * When it matches it will obtain the value and then invoke the mapper.

 *
 * @author Warwick Slade
 */
public class ValuePairAttributeMapper<T, U> extends AttributeMapper<T, U>
{
    private ModelElementSourceDestinationMapper nameModelElementMapper;
    private ModelElementSourceDestinationMapper valueModelElementMapper;

    @Override
    public void map(T input, U providedOutput)
    {
        boolean attributeSet = false;

        input = retrieveInput(getSourceModelElement(), input);

        if (input == null)
        {
            throw new AttributeNotFoundException("Source Model Element input class is null - no attributes to set");
        }

        if (!(input instanceof Collection<?>))
        {
            throw new UnexpectedException("The input must be of type collection:" + input.getClass().getName());
        }

        ModelElement destination = getDestinationModelElement();
        U output = providedOutput;
        U destinationOutput = null;
        String attributeName = null;

        if (destination != null)
        {
            attributeName =  ModelElementMapperHelper.getPropertyNameFromElement(getDestinationModelElement());
            output = destinationOutput = (U)objectTypeMapper.getAttributeFromObject(providedOutput, attributeName);
        }

        Collection<T> collection = (Collection<T>)input;

        // For each item in the input collection
        for (T item : collection)
        {
            // Check to find a mapper
            for (Mapper mapper : getModelElementMappers())
            {
                // Find the name
                String mapperAttributeName = ((Name)mapper).getName();
                Object name = nameModelElementMapper.getAttributeFromObject(sourceModelElements, item);

                if (!mapperAttributeName.equals(name))
                {
                    // This item does not belong to the mapper
                    continue;
                }
                if (destinationOutput instanceof Collection)
                {
                    // When the output is a list we instantiate a new value, but only add it to the list if it is mapped.
                    output = ModelElementMapperHelper.instantiateObjectFromModel(objectTypeMapper,
                            getDestinationModelElement(), store, providedOutput, attributeName);
                }

                try
                {
                    Object value = valueModelElementMapper.getAttributeFromObject(sourceModelElements, item);

                    mapper.map(value, output);

                    if (destinationOutput instanceof Collection)
                    {
                        ((Collection)destinationOutput).add(output);
                    }
                    attributeSet = true;
                }
                catch (AttributeNotFoundException e)
                {
                }
                // No more mappers
                break;
            }
        }

        if (!attributeSet)
        {
            throw new AttributeNotFoundException(String.format("No attributes have been found for mapping input %s to output %s",
                    input.getClass().getName(), providedOutput.getClass().getName()));
        }
    }

    public void setNameModelElementMapper(ModelElementSourceDestinationMapper nameModelElementMapper)
    {
        this.nameModelElementMapper = nameModelElementMapper;
    }

    public void setValueModelElementMapper(ModelElementSourceDestinationMapper valueModelElementMapper)
    {
        this.valueModelElementMapper = valueModelElementMapper;
    }
}
